@@ -1,28 +1,15 @@
 /*
- * ZeroTier One - Network Virtualization Everywhere
- * Copyright (C) 2011-2018  ZeroTier, Inc.  https://www.zerotier.com/
+ * Copyright (c)2019 ZeroTier, Inc.
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Use of this software is governed by the Business Source License included
+ * in the LICENSE.TXT file in the project's root directory.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Change Date: 2023-01-01
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * --
- *
- * You can be released from the requirements of the license by purchasing
- * a commercial license. Buying such a license is mandatory as soon as you
- * develop commercial closed-source software that incorporates or links
- * directly against ZeroTier software without disclosing the source code
- * of your own application.
+ * On the date above, in accordance with the Business Source License, use
+ * of this software will be governed by version 2.0 of the Apache License.
  */
+/****/
 
 #ifndef ZT_INETADDRESS_HPP
 #define ZT_INETADDRESS_HPP
@@ -91,8 +78,8 @@ struct InetAddress : public sockaddr_storage
 	};
 
 	InetAddress() { memset(this,0,sizeof(InetAddress)); }
-	InetAddress(const InetAddress &a) { ZT_FAST_MEMCPY(this,&a,sizeof(InetAddress)); }
-	InetAddress(const InetAddress *a) { ZT_FAST_MEMCPY(this,a,sizeof(InetAddress)); }
+	InetAddress(const InetAddress &a) { memcpy(this,&a,sizeof(InetAddress)); }
+	InetAddress(const InetAddress *a) { memcpy(this,a,sizeof(InetAddress)); }
 	InetAddress(const struct sockaddr_storage &ss) { *this = ss; }
 	InetAddress(const struct sockaddr_storage *ss) { *this = ss; }
 	InetAddress(const struct sockaddr &sa) { *this = sa; }
@@ -108,28 +95,28 @@ struct InetAddress : public sockaddr_storage
 	inline InetAddress &operator=(const InetAddress &a)
 	{
 		if (&a != this)
-			ZT_FAST_MEMCPY(this,&a,sizeof(InetAddress));
+			memcpy(this,&a,sizeof(InetAddress));
 		return *this;
 	}
 
 	inline InetAddress &operator=(const InetAddress *a)
 	{
 		if (a != this)
-			ZT_FAST_MEMCPY(this,a,sizeof(InetAddress));
+			memcpy(this,a,sizeof(InetAddress));
 		return *this;
 	}
 
 	inline InetAddress &operator=(const struct sockaddr_storage &ss)
 	{
 		if (reinterpret_cast<const InetAddress *>(&ss) != this)
-			ZT_FAST_MEMCPY(this,&ss,sizeof(InetAddress));
+			memcpy(this,&ss,sizeof(InetAddress));
 		return *this;
 	}
 
 	inline InetAddress &operator=(const struct sockaddr_storage *ss)
 	{
 		if (reinterpret_cast<const InetAddress *>(ss) != this)
-			ZT_FAST_MEMCPY(this,ss,sizeof(InetAddress));
+			memcpy(this,ss,sizeof(InetAddress));
 		return *this;
 	}
 
@@ -137,7 +124,7 @@ struct InetAddress : public sockaddr_storage
 	{
 		if (reinterpret_cast<const InetAddress *>(&sa) != this) {
 			memset(this,0,sizeof(InetAddress));
-			ZT_FAST_MEMCPY(this,&sa,sizeof(struct sockaddr_in));
+			memcpy(this,&sa,sizeof(struct sockaddr_in));
 		}
 		return *this;
 	}
@@ -146,7 +133,7 @@ struct InetAddress : public sockaddr_storage
 	{
 		if (reinterpret_cast<const InetAddress *>(sa) != this) {
 			memset(this,0,sizeof(InetAddress));
-			ZT_FAST_MEMCPY(this,sa,sizeof(struct sockaddr_in));
+			memcpy(this,sa,sizeof(struct sockaddr_in));
 		}
 		return *this;
 	}
@@ -155,7 +142,7 @@ struct InetAddress : public sockaddr_storage
 	{
 		if (reinterpret_cast<const InetAddress *>(&sa) != this) {
 			memset(this,0,sizeof(InetAddress));
-			ZT_FAST_MEMCPY(this,&sa,sizeof(struct sockaddr_in6));
+			memcpy(this,&sa,sizeof(struct sockaddr_in6));
 		}
 		return *this;
 	}
@@ -164,7 +151,7 @@ struct InetAddress : public sockaddr_storage
 	{
 		if (reinterpret_cast<const InetAddress *>(sa) != this) {
 			memset(this,0,sizeof(InetAddress));
-			ZT_FAST_MEMCPY(this,sa,sizeof(struct sockaddr_in6));
+			memcpy(this,sa,sizeof(struct sockaddr_in6));
 		}
 		return *this;
 	}
@@ -175,10 +162,10 @@ struct InetAddress : public sockaddr_storage
 			memset(this,0,sizeof(InetAddress));
 			switch(sa.sa_family) {
 				case AF_INET:
-					ZT_FAST_MEMCPY(this,&sa,sizeof(struct sockaddr_in));
+					memcpy(this,&sa,sizeof(struct sockaddr_in));
 					break;
 				case AF_INET6:
-					ZT_FAST_MEMCPY(this,&sa,sizeof(struct sockaddr_in6));
+					memcpy(this,&sa,sizeof(struct sockaddr_in6));
 					break;
 			}
 		}
@@ -191,10 +178,10 @@ struct InetAddress : public sockaddr_storage
 			memset(this,0,sizeof(InetAddress));
 			switch(sa->sa_family) {
 				case AF_INET:
-					ZT_FAST_MEMCPY(this,sa,sizeof(struct sockaddr_in));
+					memcpy(this,sa,sizeof(struct sockaddr_in));
 					break;
 				case AF_INET6:
-					ZT_FAST_MEMCPY(this,sa,sizeof(struct sockaddr_in6));
+					memcpy(this,sa,sizeof(struct sockaddr_in6));
 					break;
 			}
 		}
@@ -388,7 +375,7 @@ struct InetAddress : public sockaddr_storage
 				break;
 			case AF_INET6:
 				r.ss_family = AF_INET6;
-				ZT_FAST_MEMCPY(reinterpret_cast<struct sockaddr_in6 *>(&r)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,16);
+				memcpy(reinterpret_cast<struct sockaddr_in6 *>(&r)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,16);
 				break;
 		}
 		return r;
@@ -537,12 +524,12 @@ struct InetAddress : public sockaddr_storage
 				return (unsigned int)(b.template at<uint16_t>(p) + 3); // other addresses begin with 16-bit non-inclusive length
 			case 0x04:
 				ss_family = AF_INET;
-				ZT_FAST_MEMCPY(&(reinterpret_cast<struct sockaddr_in *>(this)->sin_addr.s_addr),b.field(p,4),4); p += 4;
+				memcpy(&(reinterpret_cast<struct sockaddr_in *>(this)->sin_addr.s_addr),b.field(p,4),4); p += 4;
 				reinterpret_cast<struct sockaddr_in *>(this)->sin_port = Utils::hton(b.template at<uint16_t>(p)); p += 2;
 				break;
 			case 0x06:
 				ss_family = AF_INET6;
-				ZT_FAST_MEMCPY(reinterpret_cast<struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,b.field(p,16),16); p += 16;
+				memcpy(reinterpret_cast<struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,b.field(p,16),16); p += 16;
 				reinterpret_cast<struct sockaddr_in *>(this)->sin_port = Utils::hton(b.template at<uint16_t>(p)); p += 2;
 				break;
 			default:
